-- Written by Team Ulysses, http://ulyssesmod.net/
module( "Utime", package.seeall )
if not CLIENT then return end

local gpanel

function initialize()
	gpanel = vgui.Create( "UTimeMain" )
	gpanel:SetSize( 160, 40 )
	gpanel:SetPos( ScrW() - gpanel:GetWide() - 20, 20 )
end
hook.Add( "Initialize", "UtimeInitialize", initialize )

function think()
	if LocalPlayer():GetActiveWeapon():IsValid() and LocalPlayer():GetActiveWeapon():GetClass() == "gmod_camera" then
		gpanel:SetVisible( false )
	else
		gpanel:SetVisible( true )
	end
end
timer.Create( "UTimeThink", 0.6, 0, think ) 

local texGradient = surface.GetTextureID( "gui/center_gradient" )

local PANEL = {}
PANEL.Small = 40
PANEL.Large = 100
PANEL.InnerColor = Color( 250, 250, 245, 255 )
PANEL.OuterColor = Color( 0, 150, 245, 200 )

-----------------------------------------------------------
--   Name: Paint
-----------------------------------------------------------
function PANEL:Paint()	
	local wide = self:GetWide()
	local tall = self:GetTall()
	
	draw.RoundedBox( 4, 0, 0, wide, tall, self.OuterColor ) -- Draw our base
	
	surface.SetTexture( texGradient )
	surface.SetDrawColor( 255, 255, 255, 50 )
	surface.DrawTexturedRect( 0, 0, wide, tall )  -- Draw gradient overlay		
	
	if self:GetTall() > self.Small + 4 then -- Draw the white background for another player's info
		draw.RoundedBox( 4, 2, self.Small, wide - 4, tall - self.Small - 2, self.InnerColor )
		
		surface.SetTexture( texGradient )
		surface.SetDrawColor( color_white )
		surface.DrawTexturedRect( 2, self.Small, wide - 4, tall - self.Small - 2 ) -- Gradient overlay			
	end	
	
	return true
end

-----------------------------------------------------------
--   Name: Init
-----------------------------------------------------------
function PANEL:Init()
	self.Size = self.Small
	
	self.playerInfo		= vgui.Create( "UTimePlayerInfo", self )
	
	self.lblTotalTime	= vgui.Create( "Label", self )
	self.lblSessionTime	= vgui.Create( "Label", self )
	
	self.total			= vgui.Create( "Label", self )
	self.session		= vgui.Create( "Label", self )	
end

-----------------------------------------------------------
--   Name: ApplySchemeSettings
-----------------------------------------------------------
function PANEL:ApplySchemeSettings()
	self.lblTotalTime:SetFont( "Default" )
	self.lblSessionTime:SetFont( "Default" )
	self.total:SetFont( "Default" )
	self.session:SetFont( "Default" )	
	
	self.lblTotalTime:SetFGColor( color_white )
	self.lblSessionTime:SetFGColor( color_white )
	self.total:SetFGColor( color_white )
	self.session:SetFGColor( color_white )	
end

-----------------------------------------------------------
--   Name: Think
-----------------------------------------------------------
local locktime = 0
function PANEL:Think()	
	if self.Size == self.Small then
		self.playerInfo:SetVisible( false )
	else
		self.playerInfo:SetVisible( true )
	end	
	
	local tr = utilx.GetPlayerTrace( LocalPlayer(), LocalPlayer():GetCursorAimVector() )
	local trace = util.TraceLine( tr )	
	if trace.Entity and trace.Entity:IsValid() and trace.Entity:IsPlayer() then
		self.TargetSize = self.Large
		self.playerInfo:SetPlayer( trace.Entity )
		locktime = CurTime()
	end
	
	if locktime + 2 < CurTime() then
		self.TargetSize = self.Small
	end
	
	if self.Size ~= self.TargetSize then
		self.Size = math.Approach( self.Size, self.TargetSize, (math.abs( self.Size - self.TargetSize ) + 1) * 8 * FrameTime() )
		self:PerformLayout()
	end
	
	self.total:SetText( timeToStr( LocalPlayer():GetUTime() + CurTime() - LocalPlayer():GetUTimeStart() ) )
	self.session:SetText( timeToStr( CurTime() - LocalPlayer():GetUTimeStart() ) )
end

-----------------------------------------------------------
--   Name: PerformLayout
-----------------------------------------------------------
function PANEL:PerformLayout()
	self:SetSize( self:GetWide(), self.Size )
	
	self.lblTotalTime:SetSize( 52, 18 )
	self.lblTotalTime:SetPos( 8, 2 )
	self.lblTotalTime:SetText( "Total: " )	
	
	self.lblSessionTime:SetSize( 52, 18 )
	self.lblSessionTime:SetPos( 8, 20 )
	self.lblSessionTime:SetText( "Session: " )
	
	self.total:SetSize( self:GetWide() - 52, 18 )
	self.total:SetPos( 52, 2 )
	
	self.session:SetSize( self:GetWide() - 52, 18 )
	self.session:SetPos( 52, 20 )	

	self.playerInfo:SetPos( 0, 42 )
	self.playerInfo:SetSize( self:GetWide() - 8, self:GetTall() - 42 )
end

vgui.Register( "UTimeMain", PANEL, "Panel" )


local INFOPANEL = {}

-----------------------------------------------------------
--   Name: Init
-----------------------------------------------------------
function INFOPANEL:Init()
	self.lblTotalTime	= vgui.Create( "Label", self )
	self.lblSessionTime	= vgui.Create( "Label", self )
	self.lblNick		= vgui.Create( "Label", self )
	
	self.total			= vgui.Create( "Label", self )
	self.session		= vgui.Create( "Label", self )	
	self.nick			= vgui.Create( "Label", self )	
end

-----------------------------------------------------------
--   Name: SetPlayer
-----------------------------------------------------------
function INFOPANEL:SetPlayer( ply )
	self.Player = ply
end

-----------------------------------------------------------
--   Name: ApplySchemeSettings
-----------------------------------------------------------
function INFOPANEL:ApplySchemeSettings()
	self.lblTotalTime:SetFont( "Default" )
	self.lblSessionTime:SetFont( "Default" )
	self.lblNick:SetFont( "Default" )
	self.total:SetFont( "Default" )
	self.session:SetFont( "Default" )	
	self.nick:SetFont( "Default" )	
	
	self.lblTotalTime:SetFGColor( color_black )
	self.lblSessionTime:SetFGColor( color_black )
	self.lblNick:SetFGColor( color_black )
	self.total:SetFGColor( color_black )
	self.session:SetFGColor( color_black )	
	self.nick:SetFGColor( color_black )	
end

-----------------------------------------------------------
--   Name: Think
-----------------------------------------------------------
function INFOPANEL:Think()
	local ply = self.Player
	
	self.total:SetText( timeToStr( ply:GetUTime() + CurTime() - ply:GetUTimeStart() ) )
	self.session:SetText( timeToStr( CurTime() - ply:GetUTimeStart() ) )
	self.nick:SetText( ply:Nick() )
end

-----------------------------------------------------------
--   Name: PerformLayout
-----------------------------------------------------------
function INFOPANEL:PerformLayout()
	self.lblNick:SetSize( 52, 18 )
	self.lblNick:SetPos( 8, 0 )
	self.lblNick:SetText( "Nick: " )
	
	self.lblTotalTime:SetSize( 52, 18 )
	self.lblTotalTime:SetPos( 8, 18 )
	self.lblTotalTime:SetText( "Total: " )	
	
	self.lblSessionTime:SetSize( 52, 18 )
	self.lblSessionTime:SetPos( 8, 36 )
	self.lblSessionTime:SetText( "Session: " )
	
	self.nick:SetSize( self:GetWide() - 52, 18 )
	self.nick:SetPos( 52, 0 )
	
	self.total:SetSize( self:GetWide() - 52, 18 )
	self.total:SetPos( 52, 18 )
	
	self.session:SetSize( self:GetWide() - 52, 18 )
	self.session:SetPos( 52, 36 )	
end

-----------------------------------------------------------
--   Name: Paint
-----------------------------------------------------------
function INFOPANEL:Paint()
	return true
end

vgui.Register( "UTimePlayerInfo", INFOPANEL, "Panel" )
